"use strict";

/* Alpine - Copyright (c) 2009-2021 StarTex Software, LLC */

var AdminCore = AdminCore || {};

AdminCore.InstanceAddModel = function (packages) {
    var self = this;

    self.Title = "Instance Add";

    self.HasError = ko.observable(false);
    self.ErrorMessage = ko.observable(null);

    self.WizardIsLoading = ko.observable(false);
    self.Packages = ko.observableArray(packages);
    self.SelectedPackage = ko.observable(null);

    self.LoadingScreen = 'WizardLoadingScreen';
    self.CustomerInfoScreen = 'WizardCustomerInfo';
    self.InstanceProvisioningScreen = 'WizardInstanceProvisioning';
    self.SystemAccountScreen = 'WizardSystemAccount';
    self.SupportSetupScreen = 'WizardSupportSetup';

    self.CurrentWizardScreenState = ko.observable(self.CustomerInfoScreen);
    self.AttemptedNextState = ko.observable(false);
    
    self.CustomerName = ko.observable(null);
    self.CustomerHubSpotID = ko.observable(null);

    self.UrlPrefix = ko.observable(null);
    self.SiteUrl = ko.observable(null);

    self.InstanceUID = ko.observable(null);
    self.ContentDatabaseUID = ko.observable(null);

    self.FirstUserFullName = ko.observable(null);
    self.FirstUserEmailAddress = ko.observable(null);

    self.EstimatedFormsPerYear = ko.observable(null);
    self.EstimatedTotalUserContacts = ko.observable(null);
    self.EstimatedTotalHierarchyCount = ko.observable(null);
    self.EstimatedMaxActiveHierarchyCount = ko.observable(null);
    self.IsHighProfileCustomer = ko.observable(null);
    self.BypassDedicatedDatabaseChecks = ko.observable(null);

    self.AreSizingFieldsVisible = CommonCore.koComputedDeferred(function () {
        var item = self.SelectedPackage();
        if (item == null) return false;
        return item.RequireSizingEstimate == 1;
    });

    self.IsHubSpotCustomerIDRequired = CommonCore.koComputedDeferred(function () {
        var item = self.SelectedPackage();
        if (item == null) return false;
        return item.RequireHubSpotID == 1;
    });

    self.YesNoChoices = ko.observableArray([
        {Caption: 'Yes', Value: 1},
        {Caption: 'No', Value: 0}
    ]);

    self.BypassTOS = ko.observable(0);
    self.ToggleBypassTOSCheckbox = function () {
        if (self.BypassTOS() == 0) {
            self.BypassTOS(1);
        } else {
            self.BypassTOS(0);
        }
    };

    // Validation
    self._customerNameValidationMessage = ko.observable(null);
    self.CustomerNameValidationMessage = CommonCore.koComputedDeferred(function () {
        if (!CommonCore.IsNullOrEmpty(self._customerNameValidationMessage())) {
            return self._customerNameValidationMessage();
        }
        if (CommonCore.IsNullOrEmpty(self.CustomerName())) {
            return 'Customer Name is required';
        }

        if (self.CustomerName().length > 50) {
            return 'Customer Name contains too many characters';
        }

        if (self.SelectedPackage() != null && self.SelectedPackage() !== undefined && !CommonCore.IsNullOrEmpty(self.SelectedPackage().SuffixCaption)) {
            // SiteCaption msut also be under 50 characters
            // But, SiteCaption will be "CustomerName + ' (' + SuffixCaption + ')'"
            // So, if there is a SuffixCaption, we have to restrict Customer Name even more
            if (self.CustomerName().length + self.SelectedPackage().SuffixCaption.length + 3 > 50) {
                return 'Customer Name contains too many characters (it is being limited by a maximum Site Caption length, which includes the current selected Site Template\'s suffix caption)';
            }
        }
        

        return null;
    }, function(data) {
        self._customerNameValidationMessage(data);
    });

    self._customerHubSpotIDValidationMessage = ko.observable(null);
    self.CustomerHubSpotIDValidationMessage = CommonCore.koComputedDeferred(function () {
        if (self.IsHubSpotCustomerIDRequired() == 1) {
            if (CommonCore.IsNullOrEmpty(self.CustomerHubSpotID())) {
                return 'Customer HubSpot ID is required';
            }
            if (self.CustomerHubSpotID().length > 50) {
                return 'Customer HubSpot ID contains too many characters';
            }
        }
        return null;
    }, function (data) {
        self._customerHubSpotIDValidationMessage(data);
    });

    self._PackageTypeValidationMessage = ko.observable(null);
    self.PackageTypeValidationMessage = CommonCore.koComputedDeferred(function () {
        if (!CommonCore.IsNullOrEmpty(self._PackageTypeValidationMessage())) {
            return self._PackageTypeValidationMessage();
        }
        if (CommonCore.IsNullOrEmpty(self.SelectedPackage())) {
            return 'Package Type is required';
        }
        return null;
    }, function(data) {
        self._PackageTypeValidationMessage(data);
    });


    self._urlValidationMessage = ko.observable(null);
    self.UrlPrefixValidationMessage = CommonCore.koComputedDeferred(function () {

        if (!CommonCore.IsNullOrEmpty(self._urlValidationMessage())) {
            return self._urlValidationMessage();
        }

        if (CommonCore.IsNullOrEmpty(CommonCore.TrimAndToNull(self.UrlPrefix()))) {
            return 'Url Prefix is required';
        }

        if (self.UrlPrefix() != CommonCore.ReplaceAll(self.UrlPrefix(), ' ', '')) {
            return 'Url Prefix cannot contain spaces.';
        }

        if (self.UrlPrefix().indexOf('.') != -1) {
            return 'Url Prefix can not contain a period.';
        }

        if (self.UrlPrefix() != self.UrlPrefix().toLowerCase()) {
            return 'Url Prefix must be lowercase.';
        }

        if (self.UrlPrefix().length > 35) {
            // the db column is 50, but 35 leaves room for the .ehsinsight.com (or .ehspreview.com or .ehsdev.com) suffix
            return 'Url Prefix contains too many characters';
        }

        return null;

    }, function(data) {
        self._urlValidationMessage(data);
    });


    self._estimatedFormsPerYearValidationMessage = ko.observable(null);
    self.EstimatedFormsPerYearValidationMessage = CommonCore.koComputedDeferred(function () {
        if (!CommonCore.IsNullOrEmpty(self._estimatedFormsPerYearValidationMessage())) {
            return self._estimatedFormsPerYearValidationMessage();
        }

        if (self.AreSizingFieldsVisible() == 1) {
            if (CommonCore.IsNullOrEmpty(self.EstimatedFormsPerYear())) {
                return 'Estimated Forms Per Year is required.';
            }

            if (!CommonCore.IsInt32(self.EstimatedFormsPerYear())) {
                return 'Estimated Forms Per Year must be a number.';
            }

            if (CommonCore.ParseInt32(self.EstimatedFormsPerYear()) <= 0) {
                return 'Estimated Forms Per Year must be greater than 0.';
            }
        }
        return null;
    }, function (data) {
        self._estimatedFormsPerYearValidationMessage(data);
    });


    self._estimatedTotalUserContactsValidationMessage = ko.observable(null);
    self.EstimatedTotalUserContactsValidationMessage = CommonCore.koComputedDeferred(function () {
        if (!CommonCore.IsNullOrEmpty(self._estimatedTotalUserContactsValidationMessage())) {
            return self._estimatedTotalUserContactsValidationMessage();
        }

        if (self.AreSizingFieldsVisible() == 1) {
            if (CommonCore.IsNullOrEmpty(self.EstimatedTotalUserContacts())) {
                return 'Estimated Total Users and Contacts is required.';
            }

            if (!CommonCore.IsInt32(self.EstimatedTotalUserContacts())) {
                return 'Estimated Total Users and Contacts must be a number.';
            }

            if (CommonCore.ParseInt32(self.EstimatedTotalUserContacts()) <= 0) {
                return 'Estimated Total Users and Contacts must be greater than 0.';
            }
        }
        return null;
    }, function (data) {
        self._estimatedTotalUserContactsValidationMessage(data);
    });


    self._estimatedTotalHierarchyCountValidationMessage = ko.observable(null);
    self.EstimatedTotalHierarchyCountValidationMessage = CommonCore.koComputedDeferred(function () {
        if (!CommonCore.IsNullOrEmpty(self._estimatedTotalHierarchyCountValidationMessage())) {
            return self._estimatedTotalHierarchyCountValidationMessage();
        }

        if (self.AreSizingFieldsVisible() == 1) {
            if (CommonCore.IsNullOrEmpty(self.EstimatedTotalHierarchyCount())) {
                return 'Estimated Total Business Entities is required.';
            }

            if (!CommonCore.IsInt32(self.EstimatedTotalHierarchyCount())) {
                return 'Estimated Total Business Entities must be a number.';
            }

            if (CommonCore.ParseInt32(self.EstimatedTotalHierarchyCount()) <= 0) {
                return 'Estimated Total Business Entities must be greater than 0.';
            }
        }
        return null;
    }, function (data) {
        self._estimatedTotalHierarchyCountValidationMessage(data);
    });


    self._estimatedMaxActiveHierarchyCountValidationMessage = ko.observable(null);
    self.EstimatedMaxActiveHierarchyCountValidationMessage = CommonCore.koComputedDeferred(function () {
        if (!CommonCore.IsNullOrEmpty(self._estimatedMaxActiveHierarchyCountValidationMessage())) {
            return self._estimatedMaxActiveHierarchyCountValidationMessage();
        }

        if (self.AreSizingFieldsVisible() == 1) {
            if (CommonCore.IsNullOrEmpty(self.EstimatedMaxActiveHierarchyCount())) {
                return 'Estimated Max Active Business Entities is required';
            }

            if (!CommonCore.IsInt32(self.EstimatedMaxActiveHierarchyCount())) {
                return 'Estimated Max Active Business must be a number.';
            }

            if (CommonCore.ParseInt32(self.EstimatedMaxActiveHierarchyCount()) <= 0) {
                return 'Estimated Max Active Business must be greater than 0.';
            }
        }
        return null;
    }, function (data) {
        self._estimatedMaxActiveHierarchyCountValidationMessage(data);
    });

    self._isHighProfileCustomerValidationMessage = ko.observable(null);
    self.IsHighProfileCustomerValidationMessage = CommonCore.koComputedDeferred(function () {
        if (!CommonCore.IsNullOrEmpty(self._isHighProfileCustomerValidationMessage())) {
            return self._isHighProfileCustomerValidationMessage();
        }
        if(self.AreSizingFieldsVisible()) {
            if (CommonCore.IsNullOrEmpty(self.IsHighProfileCustomer())) {
                return "You must answer 'Yes' or 'No'";
            }
        }
        return null;
    }, function (data) {
        self._isHighProfileCustomerValidationMessage(data);
    });

    self._bypassDedicatedDatabaseChecksValidationMessage = ko.observable(null);
    self.BypassDedicatedDatabaseChecksValidationMessage = CommonCore.koComputedDeferred(function () {
        if (!CommonCore.IsNullOrEmpty(self._bypassDedicatedDatabaseChecksValidationMessage())) {
            return self._bypassDedicatedDatabaseChecksValidationMessage();
        }

        if (self.AreSizingFieldsVisible()) {
            if (CommonCore.IsNullOrEmpty(self.BypassDedicatedDatabaseChecks())) {
                return "You must answer 'Yes' or 'No'";
            }
        }
        return null;
    }, function (data) {
        self._IsHighProfileCustomerValidationMessage(data);
    });


    self._firstUserFullNameValidationMessage = ko.observable(null);
    self.FirstUserFullNameValidationMessage = CommonCore.koComputedDeferred(function () {
        if (!CommonCore.IsNullOrEmpty(self._firstUserFullNameValidationMessage())) {
            return self._firstUserFullNameValidationMessage();
        }
        if (CommonCore.IsNullOrEmpty(self.FirstUserFullName()) && !CommonCore.IsNullOrEmpty(self.FirstUserEmailAddress())) {
            return 'Full Name is required when Email Address is specified.';
        }

        if (CommonCore.DefaultIfNullOrEmpty(self.FirstUserFullName(),'').length > 100) {
            return 'Full Name contains too many characters';
        }
        return null;
    }, function(data) {
        self._firstUserFullNameValidationMessage(data);
    });

    self._firstUserEmailAddressValidationMessage = ko.observable(null);
    self.FirstUserEmailAddressValidationMessage = CommonCore.koComputedDeferred(function () {
        if (!CommonCore.IsNullOrEmpty(self._firstUserEmailAddressValidationMessage())) {
            return self._firstUserEmailAddressValidationMessage();
        }
        if (CommonCore.IsNullOrEmpty(self.FirstUserEmailAddress()) && !CommonCore.IsNullOrEmpty(self.FirstUserFullName())) {
            return 'Email Address is required when Full Name is specified.';
        }

        if (CommonCore.DefaultIfNullOrEmpty(self.FirstUserEmailAddress(),'').length > 100) {
            return 'Email Address contains too many characters';
        }
        return null;
    }, function(data) {
        self._firstUserEmailAddressValidationMessage(data);
    });

    self.OtherValidationMessage = ko.observable(null);
    
    
    self.ClearServerValidationErrors = function() {

        if (!CommonCore.IsNullOrEmpty(self._PackageTypeValidationMessage())) {
            self._PackageTypeValidationMessage(null);
        }

        if (!CommonCore.IsNullOrEmpty(self._urlValidationMessage())) {
            self._urlValidationMessage(null);
        }

        if (!CommonCore.IsNullOrEmpty(self._customerNameValidationMessage())) {
            self._customerNameValidationMessage(null);
        }

        if (!CommonCore.IsNullOrEmpty(self._customerHubSpotIDValidationMessage())) {
            self._customerHubSpotIDValidationMessage(null);
        }

        if (!CommonCore.IsNullOrEmpty(self._estimatedFormsPerYearValidationMessage())) {
            self._estimatedFormsPerYearValidationMessage(null);
        }

        if (!CommonCore.IsNullOrEmpty(self._estimatedTotalUserContactsValidationMessage())) {
            self._estimatedTotalUserContactsValidationMessage(null);
        }

        if (!CommonCore.IsNullOrEmpty(self._estimatedTotalHierarchyCountValidationMessage())) {
            self._estimatedTotalHierarchyCountValidationMessage(null);
        }

        if (!CommonCore.IsNullOrEmpty(self._estimatedMaxActiveHierarchyCountValidationMessage())) {
            self._estimatedMaxActiveHierarchyCountValidationMessage(null);
        }

        if (!CommonCore.IsNullOrEmpty(self._isHighProfileCustomerValidationMessage())) {
            self._isHighProfileCustomerValidationMessage(null);
        }

        if (!CommonCore.IsNullOrEmpty(self._bypassDedicatedDatabaseChecksValidationMessage())) {
            self._bypassDedicatedDatabaseChecksValidationMessage(null);
        }

        if (!CommonCore.IsNullOrEmpty(self.OtherValidationMessage())) {
            self.OtherValidationMessage(null);
        }
    };




    self.ProcessingStatus = ko.observable(null);
    self.ProgressMax = ko.observable(null);
    self.ProgressNow = ko.observable(null);
    self.PercentDone = CommonCore.koComputedDeferred(function() {
        var percent = (self.ProgressNow()/self.ProgressMax()) * 100;
        return percent + '%';
    });




    // Wizard
    self.IsCustomerInfoScreenValid = CommonCore.koComputedDeferred(function () {
        // Should return true when Customer Info screen is valid
        var isInvalid = false;

        isInvalid = isInvalid || !CommonCore.IsNullOrEmpty(self.CustomerNameValidationMessage());
        isInvalid = isInvalid || !CommonCore.IsNullOrEmpty(self.CustomerHubSpotIDValidationMessage());
        isInvalid = isInvalid || !CommonCore.IsNullOrEmpty(self.PackageTypeValidationMessage());
        isInvalid = isInvalid || !CommonCore.IsNullOrEmpty(self.UrlPrefixValidationMessage());
        isInvalid = isInvalid || !CommonCore.IsNullOrEmpty(self.EstimatedFormsPerYearValidationMessage());
        isInvalid = isInvalid || !CommonCore.IsNullOrEmpty(self.EstimatedTotalUserContactsValidationMessage());
        isInvalid = isInvalid || !CommonCore.IsNullOrEmpty(self.EstimatedTotalHierarchyCountValidationMessage());
        isInvalid = isInvalid || !CommonCore.IsNullOrEmpty(self.EstimatedMaxActiveHierarchyCountValidationMessage());
        isInvalid = isInvalid || !CommonCore.IsNullOrEmpty(self.IsHighProfileCustomerValidationMessage());
        isInvalid = isInvalid || !CommonCore.IsNullOrEmpty(self.BypassDedicatedDatabaseChecksValidationMessage());
        isInvalid = isInvalid || !CommonCore.IsNullOrEmpty(self.FirstUserFullNameValidationMessage());
        isInvalid = isInvalid || !CommonCore.IsNullOrEmpty(self.FirstUserEmailAddressValidationMessage());
        isInvalid = isInvalid || !CommonCore.IsNullOrEmpty(self.OtherValidationMessage());

        return !isInvalid;
    });

    self.AfterValidConfirmation = function() {

        var bypassValue = self.BypassDedicatedDatabaseChecks() != null ? self.BypassDedicatedDatabaseChecks().Value : null;

        if (bypassValue == 1) {
            if (confirm('Are you sure you wish to bypass dedicated database validation?')) {
                return true;
            } else {
                return false;
            }
        }
        return true;

    };

    self.IsInstanceProvisioned = ko.observable(false);

    self.NextWizardState = function () {

        if (self.AttemptedNextState()) {
            self.ClearServerValidationErrors();
        }

        if (!self.IsCustomerInfoScreenValid()) {

            // Validation Error
            self.AttemptedNextState(true);
            self.CurrentWizardScreenState(self.WizardScreenState());

        }
        else if (self.CurrentWizardScreenState() == self.CustomerInfoScreen) {

            if (self.AfterValidConfirmation()) {
                self.AddInstanceAndProvision();
                self.CurrentWizardScreenState(self.WizardScreenState());
                self.AttemptedNextState(false);
            }

        } else {

            self.CurrentWizardScreenState(self.WizardScreenState());
            self.AttemptedNextState(false);
            
        }
    };

    self.WizardScreenState = CommonCore.koComputedDeferred(function () {
        if (self.WizardIsLoading()) {
            // Loading Screen
            return self.LoadingScreen;
        } else if (!self.IsCustomerInfoScreenValid()) {
            // Customer Info Screen
            return self.CustomerInfoScreen;
        } else if (!self.IsInstanceProvisioned()) {
            // Instance Deployment Screen
            return self.InstanceProvisioningScreen;
        } else {
            // System Account Screen
            return self.SystemAccountScreen;
        }
    });

    self.PollProvisioning = function (instanceUID, provisioningUID, siteContentDatabaseUID) {

        var packageData = {
            provisioningUID: provisioningUID,
            siteContentDatabaseUID: siteContentDatabaseUID
        };

        var provisionInterval = setInterval(function () {

            CommonCore.PostData('/admin/instances/add/status', ko.toJSON(packageData), function (data) {

                var o = jQuery.parseJSON(data);

                if (o.ResultCode === 'OK') {

                    if (!CommonCore.IsNullOrEmpty(o.CurrentStatus)) {

                        var currentStatus = o.CurrentStatus;

                        //update status if it has changed (Queued -> Processing -> Error, etc.)
                        if (currentStatus.Status != self.ProcessingStatus()) {
                            self.ProcessingStatus(currentStatus.Status);
                        }

                        if (currentStatus.TotalSteps != self.ProgressMax) {
                            self.ProgressMax(currentStatus.TotalSteps);
                        }

                        if (currentStatus.FinishedSteps != self.ProgressNow) {
                            self.ProgressNow(currentStatus.FinishedSteps);
                        }

                        if (currentStatus.Status == 'Complete' || currentStatus.Status == 'CompleteWithWarning' || currentStatus.Status == 'Cancel') {
                            clearInterval(provisionInterval);
                            self.IsInstanceProvisioned(true);
                        }

                        if (self.HasError() && (currentStatus.Status == "Queued" || currentStatus.Status == "InProgress")) {
                            // we recovered from an error condition (likely a network communication issue)
                            self.HasError(false);
                            self.ErrorMessage(null);
                        }

                    }

                } else {

                    self.HasError(true);
                    self.ErrorMessage('Error retrieving package status: ' + o.Description);

                }
            }, function (message) {
                //  client timeout

                self.HasError(true);
                self.ErrorMessage('Error retrieving package status: ' + message);

            });

        }, 3000);

    };

    self.AddInstanceAndProvision = function () {

        self.WizardIsLoading(true);

        var instancePost = {
            CustomerName: self.CustomerName(),
            CustomerHubSpotID: CommonCore.TrimAndToNull(self.CustomerHubSpotID()),
            UrlPrefix: CommonCore.TrimAndToNull(self.UrlPrefix()),
            PackageUID: self.SelectedPackage().RowUID,
            FirstUserFullName: self.FirstUserFullName(),
            FirstUserEmailAddress: self.FirstUserEmailAddress(),
            BypassTOS: self.BypassTOS(),
            EstimatedFormsPerYear: self.EstimatedFormsPerYear(),
            EstimatedTotalUserContacts: self.EstimatedTotalUserContacts(),
            EstimatedTotalHierarchyCount: self.EstimatedTotalHierarchyCount(),
            EstimatedMaxActiveHierarchyCount: self.EstimatedMaxActiveHierarchyCount(),
            IsHighProfileCustomer: self.IsHighProfileCustomer() != null ? self.IsHighProfileCustomer().Value : null,
            BypassDedicatedDatabaseChecks: self.BypassDedicatedDatabaseChecks() != null ? self.BypassDedicatedDatabaseChecks().Value : null
        };

        CommonCore.PostData('/admin/instances/add/create', ko.toJSON(instancePost), function (data) {

            var o = CommonCore.ParseJson(data);

            if (o.ResultCode === 'OK') {
                self.WizardIsLoading(false);
                self.NextWizardState();
                self.SiteUrl(o.SiteUrl);
                self.InstanceUID(o.InstanceUID);
                self.ContentDatabaseUID(o.SiteContentDatabaseUID);

                self.PollProvisioning(o.InstanceUID, o.ProvisioningUID, o.SiteContentDatabaseUID);

            } else if (o.ResultCode == 'Validation') {

                // add server validated messages
                for (var i = 0; i< o.Messages.length; i++) {
                    if (o.Messages[i].ValidationKey == 'PackageType') {
                        self.PackageTypeValidationMessage(o.Messages[i].Message);
                    } else if (o.Messages[i].ValidationKey == 'UrlPrefix') {
                        self.UrlPrefixValidationMessage(o.Messages[i].Message);
                    } else if (o.Messages[i].ValidationKey == 'EstimatedFormsPerYear') {
                        self.EstimatedFormsPerYearValidationMessage(o.Messages[i].Message);
                    } else if (o.Messages[i].ValidationKey == 'EstimatedTotalUserContacts') {
                        self.EstimatedTotalUserContactsValidationMessage(o.Messages[i].Message);
                    } else if (o.Messages[i].ValidationKey == 'EstimatedTotalHierarchyCount') {
                        self.EstimatedTotalHierarchyCountValidationMessage(o.Messages[i].Message);
                    } else if (o.Messages[i].ValidationKey == 'EstimatedMaxActiveHierarchyCount') {
                        self.EstimatedMaxActiveHierarchyCountValidationMessage(o.Messages[i].Message);
                    } else if (o.Messages[i].ValidationKey == 'IsHighProfileCustomer') {
                        self.IsHighProfileCustomerValidationMessage(o.Messages[i].Message);
                    } else {
                        self.OtherValidationMessage(o.Messages[i].Message);
                    }
                }

                self.WizardIsLoading(false);
                self.NextWizardState();

            } else {
                // Error
                self.WizardIsLoading(false);
                self.HasError(true);
                self.ErrorMessage(o.Description);
            }
        }, function (message) {
            //Error
            self.WizardIsLoading(false);
            self.HasError(true);
            self.ErrorMessage(message);
            self.CurrentWizardScreenState(self.CustomerInfoScreen);
        });
    };
};

