﻿"use strict";

/* Alpine - Copyright (c) 2009-2021 StarTex Software, LLC */

var AdminCore = AdminCore || {};

AdminCore.InstanceFlagEditModel = function (instanceJson) {
    var self = this;

    self.IsLoading = ko.observable(true);

    self.InstanceName = instanceJson.InstanceName;
    self.InstanceUID = instanceJson.InstanceUID;
    self.SiteContentDatabaseUID = instanceJson.SiteContentDatabaseUID;

    self.PreviewFlagValues = null;      // saved copy of flag values when Preview is clicked, then used later by the Save button

    self.UserCount = instanceJson.UserCount;
    self.IsDevOpsUser = instanceJson.IsDevOpsUser;

    self.PageMode = ko.observable(self.UserCount == 0 ? 'Single' : 'Edit');      // Edit, Preview, Single

    self.CurrentPreviewTab = ko.observable('Summary');

    if (CommonCore.IsNullOrEmpty(instanceJson.FlagDefs)) {
        self.FlagDefs = [];
    } else {
        self.FlagDefs = CommonCore.DefaultIfNullOrEmpty(instanceJson.FlagDefs.Categories, []);
    }

    self.FlagModels = ko.observableArray([]);

    self.UserTemplateAccounts = ko.observableArray(instanceJson.UserTemplateAccounts);
    
    self.ErrorMessages = ko.observableArray();

    self.Title = ko.observable(instanceJson.InstanceName);

    self.SearchText = ko.observable('');
    self.SearchTextDelayed = ko.computed(function() { return self.SearchText(); }).extend({throttle: 500});

    self.CategorySearchText = ko.observable('');
    self.CategorySearchTextDelayed = ko.computed(function() { return self.CategorySearchText(); }).extend({throttle: 500});

    self.IsAfterSave = ko.observable(false);
    
    self.FlagDefTree = [];

    self.LoadFlags = function (cat, list, parentFlag, indentLevel) {

        if (list !== undefined && list != null) {
            for (var i = 0; i < list.length; i++) {
                var def = list[i];
                def.IndentLevel = indentLevel;

                var model = new AdminCore.FlagModel(def, self, instanceJson, parentFlag);
                cat.Flags.push(model);
                self.FlagModels.push(model);

                self.LoadFlags(cat, def.ChildFlags, model, indentLevel + 1);
            }
        }

    };


    // load categories and flags
    for (var i=0; i < self.FlagDefs.length;i++) {
        var def = self.FlagDefs[i];
        var catCaption = CommonCore.FindByLanguageCode(def.InstanceFlagCategoryDefCaptionList, 'en', 'en').Caption;
        var cat = {Category: catCaption, Flags: []};
        self.FlagDefTree.push(cat);
        self.LoadFlags(cat, def.InstanceFlagDefList, null, 0);
    }



    self.FilterCategoriesByPreview = function() {
        var ret = [];
        for (var i=0; i<self.FlagDefTree.length; i++) {
            var cat = self.FlagDefTree[i];
            for (var j=0; j<cat.Flags.length; j++) {
                var def = cat.Flags[j];
                if (def.PreviewIsVisible()) {
                    ret.push(cat);
                    break;
                }
            }
        }
        return ret;
    };


    self.FilterFlagsByPreview = function(rows) {
        var ret = [];
        for (var i=0; i<rows.length; i++) {
            var def = rows[i];
            if (def.PreviewIsVisible()) {
                ret.push(def);
            }
        }
        return ret;
    };

    self.FilterCategoriesAndFlagsByPreview = function() {
        var ret = [];
        for (var i=0; i<self.FlagDefTree.length; i++) {
            var cat = self.FlagDefTree[i];
            for (var j=0; j<cat.Flags.length; j++) {
                var def = cat.Flags[j];
                if (def.PreviewResult() != 'NONE') {
                    ret.push(def);
                }
            }
        }
        return ret;
    };


    self.PreviewWarnCount = function() {
        var cnt = 0;
        for (var i=0; i<self.FlagDefTree.length; i++) {
            var cat = self.FlagDefTree[i];
            for (var j=0; j<cat.Flags.length; j++) {
                var def = cat.Flags[j];
                if (def.PreviewResult() == 'WARN') {
                    cnt++;
                }
            }
        }
        return cnt;
    };

    self.PreviewBlockCount = function() {
        var cnt = 0;
        for (var i=0; i<self.FlagDefTree.length; i++) {
            var cat = self.FlagDefTree[i];
            for (var j=0; j<cat.Flags.length; j++) {
                var def = cat.Flags[j];
                if (def.PreviewResult() == 'BLOCK') {
                    cnt++;
                }
            }
        }
        return cnt;
    };


    self.FilterCategoriesByKeywords = function() {
        return CommonCore.FilterRowsByKeywords(self.FlagDefTree, self.CategorySearchTextDelayed(), ['Category']);
    };


    self.FilterFlagsByKeywords = function(rows) {
        return CommonCore.FilterRowsByKeywords(rows, self.SearchTextDelayed(), ['Caption','FlagKey']);
    };

    self.IsSectionVisible = function(rows) {
        var frows = self.FilterFlagsByKeywords(rows);
        for (var i=0; i<frows.length; i++) {
            if (frows[i].IsVisible()) return true;
        }
        return false;
    };


    self.GetFeatureFlagValue = function (flagKey) {
        var flags = self.FlagValues();
        for (var i = 0; i < flags.length; i++) {
            var flag = flags[i];
            if (flag.FlagKey == flagKey) {
                return flag.FlagValue;
            }
        }
        return null;
    };

    self.FlagValues = CommonCore.koComputedDeferred( function() {
        var flagValues = [];
        for (var i = 0; i < self.FlagModels().length; i++) {
            var m = self.FlagModels()[i];
            var val = m.Value();
            flagValues.push({ RowUID: m.RowUID, ChangeToken: m.ChangeToken, FlagKey: m.Def.FlagKey, FlagValue: val });
        }
        return flagValues;
    });

    self.ValidationMessages = CommonCore.koComputedDeferred(function () {
        var messages = [];
        for (var i = 0; i < self.FlagModels().length; i++) {
            var m = self.FlagModels()[i];
            if (!m.IsValid()) {
                messages.push(m.ValidationMessage);
            }
        }
        return messages;
    });


    self.Cancel = function() {
        location.href = "/admin/instances/view/" + self.InstanceUID + "/" + self.SiteContentDatabaseUID;
    };

    self.Back = function() {
        self.PageMode('Edit');
    };

    self.CopyPreviewDetailsToClipboard = function() {

        var aux = document.createElement("div");
        aux.setAttribute("contentEditable", true);
        aux.innerHTML = document.getElementById('previewDetails').outerHTML;
        aux.setAttribute("onfocus", "document.execCommand('selectAll',false,null)");
        document.body.appendChild(aux);
        aux.focus();
        document.execCommand("copy");
        document.body.removeChild(aux);

    };

    self.Preview = function() {

        ShowAdminLoadingModal();

        var validationMessages = self.ValidationMessages();
        if (validationMessages.length > 0) {
            HideAdminLoadingModal();
            self.IsAfterSave(true);
            return;
        }

        // save off the current flag values (will be used now to POST to preview and again later to POST to save)
        self.PreviewFlagValues = self.FlagValues();

        // clear preview results from flag models
        for (var i=0; i < self.FlagModels().length;i++) {
            var def = self.FlagModels()[i];
            def.ClearPreview();
        }

        var instance = {
            SiteContentDatabaseUID: self.SiteContentDatabaseUID,
            InstanceUID: self.InstanceUID,
            Flags: self.PreviewFlagValues
        };

        var post = ko.mapping.toJS(instance);

        self.ErrorMessages.removeAll();

        CommonCore.PostData('/admin/instances/flagpreview', ko.toJSON(post), function (data) {

            var o = $.parseJSON(data);

            if (o.ResultCode === 'OK') {

                var results = o.Results;

                // apply preview results to flag models
                for (var i=0; i < self.FlagModels().length;i++) {
                    var def = self.FlagModels()[i];
                    def.LoadPreviewResult(results);
                }

                // switch to preview view
                self.CurrentPreviewTab('Summary');
                self.PageMode('Preview');

                HideAdminLoadingModal();

            } else if (o.ResultCode === 'Validation') {

                HideAdminLoadingModal();

                var messages = o.Messages;
                CommonCore.forEach(messages, function (message) {
                    self.ErrorMessages.push(message.Message);
                });

            } else {

                HideAdminLoadingModal();
                self.ErrorMessages.push(o.Description + ' - ' + o.Details);

            }

        }, function (message) {

            HideAdminLoadingModal();

            //error
            self.ErrorMessages.push('Error previewing  - ' + message);

        }, 90 * 1000);

    };



    self.Save = function () {

        ShowAdminLoadingModal();

        var validationMessages = self.ValidationMessages();
        if (validationMessages.length > 0) {
            HideAdminLoadingModal();
            self.IsAfterSave(true);
            return;
        }

        var flagValues = self.PageMode() == 'Single' ? self.FlagValues() : self.PreviewFlagValues;
       
        var instance = {
            SiteContentDatabaseUID: self.SiteContentDatabaseUID,
            InstanceUID: self.InstanceUID,            
            Flags: flagValues
        };

        var post = ko.mapping.toJS(instance);

        self.ErrorMessages.removeAll();

        CommonCore.PostData('/admin/instances/flagsave', ko.toJSON(post), function (data) {

            var o = $.parseJSON(data);

            if (o.ResultCode === 'OK') {

                location.href = "/admin/instances/view/" + self.InstanceUID + "/" + self.SiteContentDatabaseUID;

            } else if (o.ResultCode === 'Validation') {

                HideAdminLoadingModal();

                var messages = o.Messages;
                CommonCore.forEach(messages, function (message) {
                    self.ErrorMessages.push(message.Message);
                });

            } else {

                HideAdminLoadingModal();

                self.ErrorMessages.push(o.Description + ' - ' + o.Details);
            }
            
        }, function (message) {

            HideAdminLoadingModal();

            //error
            self.ErrorMessages.push('Error saving  - ' + message);

        }, 90 * 1000);
    };


    setTimeout(function() {
        self.IsLoading(false);
    },0);

};


AdminCore.GetDefaultValueForFlag = function (def) {
    if (def.FlagDataType == "String")
        return "";
    if (def.FlagDataType == "Integer" || def.FlagDataType == "Decimal2")
        return 0;
    if (def.FlagDataType == "OnOff")
        return "0";
    else return "";
};

AdminCore.FlagModel = function(def, parentModel, instanceJson, parentFlag) {

    var self = this;
  
    self.Def = def;

    self.ParentModel = parentModel;

    self.ParentFlag = parentFlag;

    self.Caption = CommonCore.FindByLanguageCode(def.InstanceFlagDefCaptionList, 'en', 'en').Caption;
    self.FlagKey = def.FlagKey;

    self.HelpText = CommonCore.FindByLanguageCode(def.InstanceFlagDefCaptionList, 'en', 'en').HelpText;
    self.Notes = CommonCore.FindByLanguageCode(def.InstanceFlagDefCaptionList, 'en', 'en').Notes;

    self.ValidationMessage = CommonCore.FindByLanguageCode(def.InstanceFlagDefCaptionList, 'en', 'en').ValidationMessage;


    self.PreviewOriginalValue = ko.observable(null);
    self.PreviewIsVisible = ko.observable(false);
    self.PreviewResult = ko.observable(null);           // NONE, OK, WARN, BLOCK
    self.PreviewMessages = ko.observableArray();

    self.ClearPreview = function() {
        self.PreviewOriginalValue(null);
        self.PreviewIsVisible(false);
        self.PreviewResult('NONE');
        self.PreviewMessages([]);
    };

    self.LoadPreviewResult = function(results) {
        for (var i=0; i<results.length; i++) {
            var result = results[i];
            if (result.FlagKey == self.Def.FlagKey) {
                self.SetPreviewVisible();
                self.PreviewResult(result.Result);
                self.PreviewMessages(result.Messages);
                self.PreviewOriginalValue(result.FlagValue);
            }
        }
    };

    self.SetPreviewVisible = function() {
        self.PreviewIsVisible(true);
        if (self.ParentFlag != null) {
            self.ParentFlag.SetPreviewVisible();
        }
    };

    self.PreviewOriginalValueCaption = CommonCore.koComputedDeferred(function() {
        if (self.Def.FlagDataType == 'OnOff') {
            if (self.PreviewOriginalValue() == '1') {
                return 'On';
            } else {
                return 'Off';
            }
        }
        return self.PreviewOriginalValue();
    });

    self.PreviewValueCaption = CommonCore.koComputedDeferred(function() {
        if (self.Def.FlagDataType == 'OnOff') {
            if (self.Value() == '1') {
                return 'On';
            } else {
                return 'Off';
            }
        }
        return self.Value();
    });


    self.GetChangeIconTitle = function(target) {

        var code = target == 'On' ? self.GetChangeIconOn() : self.GetChangeIconOff();        // use cached result

        var utarget = target.toUpperCase();

        switch (code) {
            case 'Always': return 'This flag can be turned ' + utarget + ' without issues.';
            case 'Never': return 'This flag cannot be turned ' + utarget + ' (without data migration).';
            case 'BlockingCondition': return 'This flag could be blocked from being turned ' + utarget + ' (without data migration).';
            case 'Condition': return 'This flag can be turned ' + utarget + ', but might have warnings.';
            default: return 'This flag cannot be turned ' + utarget + ' (without data migration).';
        }
    };

    self.GetChangeIconCss = function(target) {

        var code = target == 'On' ? self.GetChangeIconOn() : self.GetChangeIconOff();        // use cached result

        switch (code) {
            case 'Always': return 'fa fa-check fa-fw';
            case 'Never': return 'fa fa-times fa-fw';
            case 'BlockingCondition': return 'fa fa-exclamation-circle fa-fw';
            case 'Condition': return 'fa fa-exclamation-triangle fa-fw';
            default: return 'fa fa-minus fa-fw';
        }
    };

    self.GetChangeIconOn = CommonCore.koComputedDeferred(function() {
        // cache result
        return self.GetChangeIcon('On');
    });

    self.GetChangeIconOff = CommonCore.koComputedDeferred(function() {
        // cache result
        return self.GetChangeIcon('Off');
    });

    self.GetChangeIcon = function(target) {
        if (self.Def.InstanceFlagDefChangeList == null) {
            return "None";
        }
        var hasNever = false;
        var hasBlockingCondition = false;
        var hasCondition = false;
        var hasAlways = false;

        for (var i=0; i<self.Def.InstanceFlagDefChangeList.length; i++) {
            var change = self.Def.InstanceFlagDefChangeList[i];

            if (change.Target == 'Change' || change.Target==target) {
                if (change.AllowedWhen == 'Condition') {
                    hasCondition = true;
                    if (change.IsBlocking == 1) {
                        hasBlockingCondition = true;
                    }
                } else if (change.AllowedWhen == 'Always') {
                    hasAlways = true;
                } else if (change.AllowedWhen == 'Never') {
                    hasNever = true;
                }
            }
        }

        if (hasNever) return 'Never';
        if (hasBlockingCondition) return 'BlockingCondition';
        if (hasCondition) return 'Condition';
        if (hasAlways) return 'Always';

        return 'None';
    };







    self._value = ko.observable(0);
    self.Value = CommonCore.koComputedDeferred(function() {

        if (!self.IsActive()) {
            return CommonCore.DefaultIfNullOrEmpty(self.Def.ValueWhenInActive, AdminCore.GetDefaultValueForFlag(self.Def)) ;
        }
        return self._value();

    }, function(value) {
        self._value(value);
    });

    self.IsVisible = CommonCore.koComputedDeferred(function() {

        CommonCore.log('FlagModel - IsVisible - ' + self.Caption);

        if (self.Def.FlagType == 'Computed') return false;
        if (self.Def.VisibilityMode == 'Hidden') return false;

        if (!self.IsActive()) return false;

        return true;
    });


    self.IsActive = CommonCore.koComputedDeferred(function() {

        CommonCore.log('FlagModel - IsActive - ' + self.Caption);

        if (!CommonCore.IsNullOrEmpty(self.Def.ActiveFormula) && !AdminCore.EvalValue(self.Def.ActiveFormula, self.ParentModel.FlagModels())) return false;
        if (!CommonCore.IsNullOrEmpty(self.Def.LicenseFormula) && !AdminCore.EvalValue(self.Def.LicenseFormula, self.ParentModel.FlagModels())) return false;

        if (self.ParentFlag != null) {
            if (!self.ParentFlag.IsActive()) return false;
            if (self.ParentFlag.Def.FlagDataType == 'OnOff' && self.ParentFlag.Value() == "0") return false;
        }

        return true;
    });


    self.IsValid = CommonCore.koComputedDeferred(function() {

        CommonCore.log('FlagModel - IsValid - ' + self.Caption);

        if (!self.IsActive()) return true;                                              // only validate flags when they are active
        if (self.Def.FlagDataType == 'OnOff' && self.Value() == 0) return true;         // only validate OnOff flags if they are ON  (this is by design to reduce formula clutter since we only want "active" flags to ever participate in validation anyway... this pattern  the validation formulas on the "dependant flags", not the "source flags")

        if (!CommonCore.IsNullOrEmpty(self.Def.ValidationFormula) && !AdminCore.EvalValue(self.Def.ValidationFormula, self.ParentModel.FlagModels())) return false;

        return true;
    });



    // initialize
    var flag = _.find(instanceJson.Flags, function(v) { return v.FlagKey == def.FlagKey });

    if (flag === undefined) {
        self.Value(CommonCore.DefaultIfNullOrEmpty(def.DefaultValue, AdminCore.GetDefaultValueForFlag(def)));
        self.RowUID = CommonCore.GenerateUID();
        self.ChangeToken = '0';
    } else {
        self.Value(CommonCore.DefaultIfNullOrEmpty(flag.FlagValue, CommonCore.DefaultIfNullOrEmpty(def.DefaultValue, AdminCore.GetDefaultValueForFlag(def))));
        self.RowUID = flag.RowUID;
        self.ChangeToken = flag.ChangeToken;
    }




};


AdminCore.EvalValue = function(expression, flags) {
    if (CommonCore.IsNullOrEmpty(expression)) return null;
    var tree = jsep(expression);
    return AdminCore.EvalTree(flags, tree);
};


AdminCore.EvalTree = function (flags, tree) {

    var left, right;

    switch (tree.type) {

        case 'BinaryExpression':
            left = AdminCore.EvalTree(flags, tree.left);
            right = AdminCore.EvalTree(flags, tree.right);
            switch (tree.operator) {

                case '==':
                    return (left == right);
                case '!=':
                    return (left != right);
                case '>':
                    return (left > right);
                case '>=':
                    return (left >= right);
                case '<':
                    return (left < right);
                case '<=':
                    return (left <= right);

                case '+':
                    return left + right;
                case '-':
                    return left - right;
                case '*':
                    return left * right;
                case '/':
                    return left / right;

                default:
                    throw new Error('AdminCore.EvalTree - Unknown BinaryExpression tree.operator: ' + tree.operator);
            }

        case 'Literal':
            return tree.value;

        case 'UnaryExpression':
            var val = AdminCore.EvalTree(flags, tree.argument);
            switch (tree.operator) {
                case '-':
                    return 0 - val;
                default:
                    throw new Error('AdminCore.EvalTree - Unknown UnaryExpression tree.operator: ' + tree.operator);
            }
      
        case 'CallExpression':
            switch (tree.callee.type) {
                case 'Identifier':

                    switch (tree.callee.name) {
                        case 'Iif':                                                        
                            return AdminCore.EvalTree(flags, AdminCore.EvalTree(flags, tree.arguments[0]) ? AdminCore.EvalTree(flags, tree.arguments[1]) : AdminCore.EvalTree(flags, tree.arguments[2]));

                        case 'FlagValue':
                            var flagKey = AdminCore.EvalTree(flags, tree.arguments[0]);
                            var flagObj = _.find(flags, function (v) { return v.FlagKey == flagKey });
                            if (flagObj !== undefined) {
                                return flagObj.Value();
                            } else {
                                throw new Error('AdminCore.EvalTree - FlagKey not found: ' + flagKey);
                            }

                        default:
                            throw new Error('AdminCore.EvalTree - Unknown CallExpression tree.callee.name: ' + tree.callee.name);
                    }                  

                default:
                    throw new Error('AdminCore.EvalTree - Unknown CallExpression tree.callee.type: ' + tree.callee.type);
            }


        case 'ConditionalExpression':
            var iif = AdminCore.EvalTree(flags, tree.test);
            return AdminCore.EvalTree(flags, iif ? tree.consequent : tree.alternate);

        case 'LogicalExpression':
            left = AdminCore.EvalTree(flags, tree.left);
            right = AdminCore.EvalTree(flags, tree.right);
            switch (tree.operator) {
                case '||':
                    return (left || right);
                case '&&':
                    return (left && right);
                default:
                    throw new Error('AdminCore.EvalTree - Unknown LogicalExpression tree.operator: ' + tree.operator);
            }

        default:
            throw new Error('AdminCore.EvalTree - Unknown tree.type: ' + tree.type);
    }

};