﻿"use strict";

/* Alpine - Copyright (c) 2009-2021 StarTex Software, LLC */

var AdminCore = AdminCore || {};

AdminCore.InstanceListModel = function (instanceDataJson) {

    var self = this;

    self.IsLoading = ko.observable(true);
    self.IsFlexLoading = ko.observable(true);

    self.InstanceData = ko.mapping.fromJS(instanceDataJson);
    

    self.StartUp = function() {

        CommonCore.DownloadData('/admin/instances/flexdata', function (data) {
            var o = CommonCore.ParseJson(data);
            switch (o.ResultCode) {
                case "OK":

                    for (var i = 0; i < o.Instances.length; i++) {
                        var flex = o.Instances[i];

                        for (var j = 0; j < self.InstanceData.Instances().length; j++) {
                            var id = self.InstanceData.Instances()[j];
                            if (id.DatabaseName() == flex.DatabaseName && id.InstanceUID() == flex.InstanceUID) {
                                id.FlexDataRowCount(flex.FlexDataRowCount);
                            }
                        }
                    }

                    self.IsFlexLoading(false);
                    break;
                default:
                    alert(o.Description + " \r\n" + o.Details);                    
                    break;
            }
        }, function(error) {
            // this will fire if you navigate away while the query is still running, so an alert is not ideal
            CommonCore.warn(error);            
        });


        // compute IsPastRetention
        for (var k = 0; k < self.InstanceData.Instances().length; k++) {
            var instance = self.InstanceData.Instances()[k];
            instance.IsPastRetention = 0;

            var dtm = instance.EndedDtm();
            if (dtm != null) {
                var instanceType = instance.InstanceType();
                var diff = moment().diff(moment(dtm), 'days');
                if (instanceType == 'DELETE' && diff > 30) {
                    instance.IsPastRetention = 1;
                }
                if (instanceType == 'CANCELLED' && diff > 90) {
                    instance.IsPastRetention = 1;
                }
            }
        }


        self.IsLoading(false);

    };

    self.HasError = ko.observable(false);
    self.ErrorMessage = ko.observable('');

    self.SearchText = ko.observable('');
    self.SearchTextDelayed = ko.computed(function() { return self.SearchText();}).extend({rateLimit:200});

    self.CurrentTab = ko.observable('Type');

    self.DistinctTypeList = CommonCore.koComputedDeferred(function() {

        
        var arr = [];
        for (var i = 0; i < self.FilteredInstanceData().length; i++) {

            var instance = self.FilteredInstanceData()[i];

            var type = "";
            var sortOrder = "";

            switch (self.CurrentTab()) {

                case 'Type': 
                    type = instance.InstanceType();
                    sortOrder = type;
                    switch (type) {
                        case 'BUILD': sortOrder = '0000'; break;
                        case 'DEV': sortOrder = '0001'; break;
                        case 'PROD': sortOrder = '0002'; break;
                        case 'SANDBOX': sortOrder = '0003'; break;
                        case 'DEMO': sortOrder = '0004'; break;
                        case 'TRIAL': sortOrder = '0005'; break;
                        case 'DELETE': sortOrder = 'ZZZZ'; break;
                    }
                    break;


                case 'Version':
                    type = instance.ConfigVersion();
                    sortOrder = type;
                    if (type.indexOf('.') >= 0) {
                        var versionSplit = type.split('.');
                        if (versionSplit.length == 4) {
                            var a = (versionSplit[0] * 10000);
                            var b = (versionSplit[1] * 100);
                            var c = (versionSplit[2] * 1);
                            var d = (versionSplit[3] / 1000000);
                            sortOrder = (a + b + c + d);
                        }
                    }
                    break;


                case 'Flex':        
                    type = (Math.ceil(instance.FlexDataRowCount() / 100000, 0) * 100) + 'K';
                    sortOrder = instance.FlexDataRowCount() / 100000;
                    break;

                case 'Date':
                    type = CommonCore.FormatData(instance.CreatedDtm(), 'YM');
                    sortOrder = type;
                    break;

                case 'Database':
                    type = instance.DatabaseName();
                    sortOrder = type;
                    break;

                case 'DatabaseServer':
                    type = instance.DatabaseServerName();
                    sortOrder = type;
                    break;

                case 'Retention':
                    if (CommonCore.IsNullOrEmpty(instance.EndedDtm())) {
                        type = 'Active';
                        sortOrder = '3';
                    } else if (instance.IsPastRetention) {
                        type = 'Ready to Delete';
                        sortOrder = '1';
                    } else {
                        type = 'In Retention Period';
                        sortOrder = '2';
                    }                                        
                    break;

            }

            var typeObj = _.find(arr, function(val) { return val.Title == type; });

            if (typeObj === undefined) {
                typeObj = { Title: type, SortOrder: sortOrder, Instances: [] };
                arr.push(typeObj);
            }

            typeObj.Instances.push(instance);

        }
        
        return arr.sort(CommonCore.SortByProperty('SortOrder'));

    });

    self.IsSectionExpanded = function (name) {
        CommonCore.Settings.Mutex();    // subscribe
        if (!CommonCore.IsNullOrEmpty(self.SearchTextDelayed())) return true;        
        return CommonCore.Settings.GetSetting("AdminPortal_InstanceList_" + self.GetSettingKey(name), '1') != '0';
    };

    self.ToggleSection = function(name, val) {
        CommonCore.Settings.SetSetting("AdminPortal_InstanceList_" + self.GetSettingKey(name), val);
    };

    self.GetSettingKey = function(name) {
        name = CommonCore.ReplaceAll(name, ' ', '');
        name = CommonCore.ReplaceAll(name, '(', '');
        name = CommonCore.ReplaceAll(name, ')', '');
        return name;
    };
   

    self.FilteredInstanceData = CommonCore.koComputedDeferred(function() {
        
        if (!CommonCore.IsNullOrEmpty(self.SearchTextDelayed())) {            
            return CommonCore.FilterRowsByKeywords(self.InstanceData.Instances(), self.SearchTextDelayed(), [
                'ConfigVersion', 'CustomerName', 'CustomerAlsoKnownAs', 'CustomerHubSpotID', 'DatabaseName', 'DatabaseServerName', 'InstanceName', 
                'InstanceType', 'InstanceUID', 'PrimaryURL', 'SiteCaption', 'ProvisioningPackageFileName'
            ]);
        }else {
            return self.InstanceData.Instances();
        }
        
    });

    self.TrimPackageFilename = function(filename) {
        if (CommonCore.IsNullOrEmpty(filename)) {
            return null;
        }
        if (CommonCore.Contains(filename, '_')) {
            return CommonCore.StringSplitTrim(filename, '_', false)[0];
        }
        return filename;
    };

    self.HasDeletedInstance = ko.computed(function () {
        var isDeleted = false;
        CommonCore.forEach(self.InstanceData.Instances(), function (instance) {
            if (instance.RowStatus() === 1) {
                isDeleted = true;
                instance.IsMapped(false);
            }
        });
        return isDeleted;
    });

    self.DeleteInstanceCheck = function (instance) {

        var deleteCheck = confirm('DANGER: Deleting an Instance can NOT be undone. Are you sure you want to complete this action?');

        if (deleteCheck) {

            instance.RowStatus(1);

            CommonCore.PostData("instances/delete", ko.toJSON(instance), function (data) {
                var o = jQuery.parseJSON(data);

                if (o.ResultCode === 'OK') {
                    self.HasError(false);
                } else {
                    self.HasError(true);
                    instance.RowStatus(1);
                    self.ErrorMessage(o.Description);
                }

            }, function (message) {
                self.HasError(true);
                self.ErrorMessage(message);
            });
        }
    };

    self.DeleteMapping = function(urlMapping) {

        CommonCore.DownloadData("urlmappings/" + urlMapping.RowUID() + "/delete", function(data) {
            var o = jQuery.parseJSON(data);

            if (o.ResultCode === 'OK') {
                var i = self.InstanceData.UnmappedUrls.indexOf(urlMapping);
                self.InstanceData.UnmappedUrls.splice(i, 1);
                self.HasError(false);
            } else {
                self.HasError(true);
                self.ErrorMessage(o.Description);
            }

        }, function(message) {
            self.HasError(true);
            self.ErrorMessage(message);
        });
    };


    // --------------------------------------

    // start up 
    self.StartUp();

};
