﻿"use strict";

/* Alpine - Copyright (c) 2009-2021 StarTex Software, LLC */

var AdminCore = AdminCore || {};

AdminCore.PackageListModel = function (packages, databases) {
    var self = this;

    self.HasError = ko.observable(null);
    self.ErrorMessage = ko.observable(null);

    self.Databases = ko.observableArray(databases);
    self.Packages = ko.observableArray(null);


    self.InitPackages = function() {
        var packageArray = [];
        if (packages) {
            for (var i = 0; i<packages.length; i++){
                var provisionPackage = new AdminCore.ProvisionPackage(databases);
                provisionPackage.LoadFromJson(packages[i]);
                packageArray.push(provisionPackage);
            }
        }
        self.Packages(packageArray);
    }(); 

    self.DeletePackage = function(provisioningPackage) {
        var post = provisioningPackage.SaveToJson();

        CommonCore.PostData('/admin/package/delete', ko.toJSON(post), function(data) {
            var o = CommonCore.ParseJson(data);
            if (o.ResultCode == "OK") {
                for (var i = 0; i < self.Packages().length; i++) {
                    if (provisioningPackage.RowUID() == self.Packages()[i].RowUID()) {
                        // delete
                        self.Packages.splice(i, 1);
                    }
                }
            } else {
                self.HasError(true);
                self.ErrorMessage('Error deleting package.');
            }
        }, function(err) {
            // error
            self.HasError(true);
            self.ErrorMessage(err);
        });
    };

    
};

AdminCore.ProvisionPackage = function(databaseList) {
    var self = this;

    self.DatabaseList = ko.observableArray(databaseList);

    self.IsDeleted = false;
    self.IsNew = false;

    self.RowUID = ko.observable(null);
    self.FileName = ko.observable(null);
    self.Caption = ko.observable(null);
    self.InstanceType = ko.observable(null);
    self.SuffixCaption = ko.observable(null);
    self.ThemeName = ko.observable(null);

    self.MajorDeleteInstanceData = ko.observable(null);
    self.RequireDedicatedDatabase = ko.observable(null);
    self.RequireSizingEstimate = ko.observable(null);
    self.RequireHubSpotID = ko.observable(null);
    self.IsSubMgrTrialPackage = ko.observable(null);
    self.IsAdpPackage = ko.observable(null);

    self.Databases = ko.observableArray([]);


    self.IsDatabaseSelected = function(database) {
        return (CommonCore.ArrayContainsWithPropertyValue(self.Databases(), 'SiteContentDatabaseUID', database.RowUID));
    };

    self.ToggleDatabase = function(database) {
        if (self.IsDatabaseSelected(database)) {
            var row = CommonCore.ArrayFindByPropertyValue(self.Databases(), 'SiteContentDatabaseUID', database.RowUID);
            self.Databases.remove(row);
            self.Databases.valueHasMutated();
        } else {
            self.Databases.push({ RowUID: CommonCore.GenerateUID(), SiteContentDatabaseUID: database.RowUID });
            self.Databases.valueHasMutated();
        }
    };

    self.SelectedDatabaseNameCsv = CommonCore.koComputedDeferred(function() {
        var arr = [];
        for (var i = 0; i < self.Databases().length; i++) {
            var dbUID = self.Databases()[i].SiteContentDatabaseUID;
            var db = CommonCore.ArrayFindByPropertyValue(self.DatabaseList(), 'RowUID', dbUID);
            if (db != null) {
                arr.push(db.Name);
            }
        }
        return CommonCore.ArrayJoin(', ', arr);

    });
    


    // Validation
    self.FileNameValidationMessage = CommonCore.koComputedDeferred(function() {
        if (CommonCore.IsNullOrEmpty(self.FileName())) {
            return CommonCore.ReplaceAll(CommonCore.LanguageCaptionCache.GetCaption('formengine_validation_isrequired', '{{ValidationCaption}} is required.'), '{{ValidationCaption}}', 'File Name');
        }

        if (self.FileName().length > 50) {
            return CommonCore.ReplaceAll(CommonCore.LanguageCaptionCache.GetCaption('formengine_validation_maxlength', '{{ValidationCaption}} contains too many characters.'), '{{ValidationCaption}}', 'File Name');
        }
    });
    self.CaptionValidationMessage = CommonCore.koComputedDeferred(function() {
        if (CommonCore.IsNullOrEmpty(self.Caption())) {
            return CommonCore.ReplaceAll(CommonCore.LanguageCaptionCache.GetCaption('formengine_validation_isrequired', '{{ValidationCaption}} is required.'), '{{ValidationCaption}}', 'Caption');
        }
        if (self.Caption().length > 50) {
            return CommonCore.ReplaceAll(CommonCore.LanguageCaptionCache.GetCaption('formengine_validation_maxlength', '{{ValidationCaption}} contains too many characters.'), '{{ValidationCaption}}', 'Caption');
        }
    });
    self.InstanceTypeValidationMessage = CommonCore.koComputedDeferred(function() {
        if (CommonCore.IsNullOrEmpty(self.InstanceType())) {
            return CommonCore.ReplaceAll(CommonCore.LanguageCaptionCache.GetCaption('formengine_validation_isrequired', '{{ValidationCaption}} is required.'), '{{ValidationCaption}}', 'Instance Type');
        }
    });
    self.ContentDatabaseValidationMessage = CommonCore.koComputedDeferred(function() {
        if (self.Databases().length == 0) {
            return CommonCore.ReplaceAll(CommonCore.LanguageCaptionCache.GetCaption('formengine_validation_isrequired', '{{ValidationCaption}} is required.'), '{{ValidationCaption}}', 'Content Database');
        }
    });
    self.SuffixCaptionValidationMessage = CommonCore.koComputedDeferred(function() {
        if (!CommonCore.IsNullOrEmpty(self.SuffixCaption())) {
            if (self.SuffixCaption().length > 50) {
                return CommonCore.ReplaceAll(CommonCore.LanguageCaptionCache.GetCaption('formengine_validation_maxlength', '{{ValidationCaption}} contains too many characters.'), '{{ValidationCaption}}', 'Suffix Caption');
            }
        }
    });

    self.IsValid = CommonCore.koComputedDeferred(function() {
        var isInvalid = false;

        isInvalid = isInvalid || !CommonCore.IsNullOrEmpty(self.FileNameValidationMessage());
        isInvalid = isInvalid || !CommonCore.IsNullOrEmpty(self.CaptionValidationMessage());
        isInvalid = isInvalid || !CommonCore.IsNullOrEmpty(self.InstanceTypeValidationMessage());
        isInvalid = isInvalid || !CommonCore.IsNullOrEmpty(self.ContentDatabaseValidationMessage());
        isInvalid = isInvalid || !CommonCore.IsNullOrEmpty(self.SuffixCaptionValidationMessage());

        return !isInvalid;
    });

    self.LoadFromJson = function(provisionPackage) {
        self.RowUID(provisionPackage.RowUID);
        self.FileName(provisionPackage.FileName);
        self.Caption(provisionPackage.Caption);
        self.InstanceType(provisionPackage.InstanceType);
        self.SuffixCaption(provisionPackage.SuffixCaption);
        self.ThemeName(provisionPackage.ThemeName);
        self.MajorDeleteInstanceData(provisionPackage.MajorDeleteInstanceData);
        self.RequireDedicatedDatabase(provisionPackage.RequireDedicatedDatabase);
        self.RequireSizingEstimate(provisionPackage.RequireSizingEstimate);
        self.RequireHubSpotID(provisionPackage.RequireHubSpotID);
        self.IsSubMgrTrialPackage(provisionPackage.IsSubMgrTrialPackage);
        self.IsAdpPackage(provisionPackage.IsAdpPackage);
        self.Databases(provisionPackage.SiteProvisioningPackageDatabaseList);

    };

    self.SaveToJson = function() {
        var provisionPackage = {};
        provisionPackage.RowUID = self.RowUID();
        provisionPackage.FileName = self.FileName();
        provisionPackage.Caption = self.Caption();
        provisionPackage.InstanceType = self.InstanceType();
        provisionPackage.SuffixCaption = self.SuffixCaption();
        provisionPackage.ThemeName = self.ThemeName();
        provisionPackage.MajorDeleteInstanceData = self.MajorDeleteInstanceData();
        provisionPackage.RequireDedicatedDatabase = self.RequireDedicatedDatabase();
        provisionPackage.RequireSizingEstimate = self.RequireSizingEstimate();
        provisionPackage.RequireHubSpotID = self.RequireHubSpotID();
        provisionPackage.IsSubMgrTrialPackage = self.IsSubMgrTrialPackage();
        provisionPackage.IsAdpPackage = self.IsAdpPackage();
        provisionPackage.SiteProvisioningPackageDatabaseList = self.Databases();
        return provisionPackage;
    };

};