﻿"use strict";

/* Alpine - Copyright (c) 2009-2021 StarTex Software, LLC */

var AdminCore = AdminCore || {};

AdminCore.ScheduledJobViewModel = function (packageDetailsJson, instanceJson) {
    var self = this;
    self.PackageDetailsJson = packageDetailsJson;
    self.InstanceJson = instanceJson;

    self.PackageStatus = ko.observable(packageDetailsJson.ProcessingStatus);
    self.ProcessingServer = ko.observable(packageDetailsJson.ProcessingServer);
    self.FileName = ko.observable(packageDetailsJson.FileName);
    self.InstanceName = ko.observable(packageDetailsJson.InstanceName);
    self.PackageDuration = ko.observable(packageDetailsJson.packageDuration);

    self.IsArchived = ko.observable(packageDetailsJson.IsArchived);

    self.CreatedDtm = ko.observable(packageDetailsJson.CreatedDtm);
    self.ProvisionSuccessDtm = ko.observable(packageDetailsJson.ProvisionSuccessDtm);


    self.DataToggle = ko.observable(false);
    self.FailedOnSubStep = ko.observable(packageDetailsJson.failedStep && packageDetailsJson.failedStep.SubStepCount != "0");

    self.PackageBytesAndJsonStatus = ko.observable(packageDetailsJson.packageBytesAndJsonStatus);

    self.ErrorMessage = ko.observable('');
    self.HasError = ko.observable(false);
    self.HasStatus = ko.observable(packageDetailsJson.currentDeployment.length > 0);

    self.IsInvalidInstance = ko.observable(false);

    self.HideFastSteps = ko.observable(false);

    self.IsDeleteInstance = ko.computed(function () {
        if (!CommonCore.IsNullOrEmpty(self.InstanceJson)) {
            if (self.InstanceJson.RowStatus == 1) {
                return true;
            }
        }
        return false;
    });

    self.GetPercentProcessed = function (subStepsProcessed, subStepCount) {
        var percent = (parseInt(subStepsProcessed, 10) / parseInt(subStepCount, 10) * 100);
        return percent + '%';
    };

    self.InProgressDeployment = ko.computed(function () {
        // note: wires up computed to refresh when invoked
        self.DataToggle();

        if (!self.HideFastSteps()) {

            return self.PackageDetailsJson.currentDeployment;
        } else {
            var arr = [];
            for (var i = 0; i < self.PackageDetailsJson.currentDeployment.length; i++) {
                var row = self.PackageDetailsJson.currentDeployment[i];
                if (row.Duration != '00:00:00' && row.Duration != '00:00:01') {
                    arr.push(row);
                }
            }
            return arr;
        }
    });

    self.ProvisioningStepMessages = ko.computed(function () {
        // note: wires up computed to refresh when invoked
        self.DataToggle();

        return self.PackageDetailsJson.provisioningStepMessages;
    });

    self.UpdatePackage = function () {

        var packageData = {
            provisioningUID: self.PackageDetailsJson.provisioningUID,
            conStr: self.PackageDetailsJson.databaseConnectionString,
            databaseUID: self.PackageDetailsJson.databaseUID,
            instanceUID: self.InstanceJson.InstanceUID
        };

        var provisionInterval = setInterval(function () {
            CommonCore.PostData('/admin/scheduledjobs/view/refresh', ko.toJSON(packageData), function (data) {

                var o = jQuery.parseJSON(data);

                if (o.ResultCode === 'OK') {

                    var status = o.Result;
                    self.PackageDetailsJson = status;
                    self.DataToggle(!self.DataToggle());
                    self.HasStatus(status.currentDeployment.length > 0);
                    self.PackageDuration(status.packageDuration);
                    self.ProvisionSuccessDtm(status.ProvisionSuccessDtm);

                    // update status if it has changed (Queued -> Processing -> Error, etc.)
                    if (status.ProcessingStatus !== self.PackageStatus()) {
                        self.PackageStatus(status.ProcessingStatus);
                    }

                    if (status.ProcessingServer !== self.ProcessingServer()) {
                        self.ProcessingServer(status.ProcessingServer);
                    }

                    if (status.ProcessingStatus === 'Complete' || status.ProcessingStatus === 'Paused' || status.ProcessingStatus === 'Pospone' || status.ProcessingStatus === 'CompleteWithWarning' || status.ProcessingStatus === 'Error' || status.ProcessingStatus === 'Cancel') {
                        clearInterval(provisionInterval);
                    }

                    if (status.ProcessingStatus === 'Error') {
                        if (self.PackageDetailsJson.failedStep && self.PackageDetailsJson.failedStep.SubStepCount != "0") {
                            self.FailedOnSubStep(true);
                        }
                    }

                    self.HasError(false);

                } else if (o.ResultCode === "InstanceNotFound") {
                    // Prevents instance finished being deleted
                    self.PackageStatus('Complete');
                    self.PackageDuration('N/A');
                    self.IsInvalidInstance(true);
                    clearInterval(provisionInterval);
                } else {
                    self.HasError(true);
                    self.ErrorMessage('Error retrieving package status: ' + o.Description);
                    clearInterval(provisionInterval);
                }
            }, function (message) {
                self.HasError(true);
                self.ErrorMessage('Error retrieving package status: ' + message);
                clearInterval(provisionInterval);
            });

        }, 3000);

    };

    self.UpdateStatus = function (newStatus) {

        var packageUpdate = {
            processingStatus: newStatus            
        };

        CommonCore.PostData('updatequeuestatus', ko.toJSON(packageUpdate), function (data) {

            var o = jQuery.parseJSON(data);

            if (o.ResultCode === 'OK') {

                self.PackageStatus(newStatus);
                self.FailedOnSubStep(false);

                if (newStatus === 'QueuedRetry' || newStatus === 'QueuedSkipStep') {
                    self.UpdatePackage();
                }
            } else {
                self.HasError(true);
                self.ErrorMessage('Update status failed, try again. Error: ' + o.Description);
            }
        }, function (message) {
            self.HasError(true);
            self.ErrorMessage('Update status failed, try again. Error: ' + message);
        });
    };

    self.GetTime = function (dateString) {
        if (CommonCore.IsNullOrEmpty(dateString))
            return "";

        return moment(dateString).format('YYYY-MM-DD HH:mm:ss');
    };

    self.GetFriendlyStatusName = function (packageStatus) {
        if (packageStatus === "CompleteWithWarning") {
            return "Complete with Warnings";
        }
        if (packageStatus === "QueuedRetry") {
            return "Retry Queued";
        }
        if (packageStatus === "QueuedSkipStep") {
            return "Skip Step Queued";
        }       
        if (packageStatus === "Cancel") {
            return "Cancelled";
        }
        if (packageStatus === "ErrorCancel") {
            return "Error With Auto-Cancel";
        }

        return packageStatus;
    };

    self.GetDurationColor = function (duration) {
        var seconds = new Date('1970-01-01T' + duration + 'Z').getTime() / 1000;        
        var value = seconds >= 60 ? 1 : seconds / 60.0;
        var hue = ((1 - value) * 60).toString(10);       // convert to yellow-red hue spectrum
        return "hsl(" + hue + ",70%,50%)";
    };


    if (self.PackageDetailsJson.ProcessingStatus === 'Queued' || self.PackageDetailsJson.ProcessingStatus === 'InProgress') {
        self.UpdatePackage();
    }

};